package com.bananahammock;

import android.graphics.drawable.Drawable;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

/**
 * @author Scott Faria - scott.faria@gmail.com
 */
public final class Utils {

	////////////////////////////////////////////////////////////
	//	Statics
	
	public static final <T> T checkNull(T o) {
		if (o == null) {
			throw new NullPointerException();
		}
		return o;
	}

    public static final void close(Closeable c) {
        if (c != null) {
            try {
                c.close();
            } catch (IOException e) {}
        }
    }

    public static final boolean deleteChildren(File dir) {
        if (dir.isDirectory() == false) {
            return false;
        }
        boolean success = false;
        for (File f : dir.listFiles()) {
            success = f.delete();
        }
        return success;
    }

    public static Drawable createDrawableFromURL(String urlString) {
        InputStream content = (InputStream) fetchContent(urlString);
        return Drawable.createFromStream(content, "src");
    }

    private static Object fetchContent(String urlString) {
        try {
            URL url = new URL(urlString);
            return url.getContent();
        } catch (Exception e) {
            return null;
        }
    }

	////////////////////////////////////////////////////////////
	//	Constructor
	
	private Utils() { /* NO */ }
}
