package com.bananahammock.activities;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Parcelable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import com.bananahammock.R;
import com.bananahammock.auth.JSoupCookieManager;
import com.bananahammock.foruminfo.ForumDescriptor;
import com.bananahammock.foruminfo.ForumParser;

import java.util.ArrayList;
import java.util.List;

/**
 * Entry point activity for D2Jsp forums browser. Will perform initial login and then list forums.
 *
 * @author Scott Faria - scott.faria@gmail.com
 */
public final class D2JspMobileActivity extends ListActivity {

    ////////////////////////////////////////////////////////////
    //	Statics

    private static final int THREAD_LIST_ID = 100;

    private static final String SUBFORUMS_KEY = "subForums";
    public static final String FORUM_URL_KEY = "forumUrl";
    public static final String THREAD_URL_KEY = "threadUrl";
    public static final String THREAD_NAME_KEY = "threadName";

    ////////////////////////////////////////////////////////////
    //	Overridden

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        JSoupCookieManager.init(getApplicationContext());
        if (JSoupCookieManager.instance().hasValidCookies() == false) {
            Intent intent = new Intent(D2JspMobileActivity.this, LoginActivity.class);
            startActivityForResult(intent, THREAD_LIST_ID);
        } else {
            final ForumDescriptorAdapter forumDescriptorAdapter = new ForumDescriptorAdapter(
                    this,
                    R.layout.list_item,
                    new ArrayList<ForumDescriptor>()
            );

            setListAdapter(forumDescriptorAdapter);

            //start up the busy indicator
            final ProgressDialog busyIndicator = ProgressDialog.show(this, "Please wait...", "Retrieving forum information...", true);

            //populate and inform the client we are waiting on it
            new AsyncTask<Void, Integer, ForumDescriptor[]>() {
                @Override
                protected ForumDescriptor[] doInBackground(Void... voids) {
                    return getViewableForums();
                }

                @Override
                protected void onPostExecute(ForumDescriptor[] forumDescriptors) {
                    forumDescriptorAdapter.notifyDataSetInvalidated();
                    for (ForumDescriptor fd : forumDescriptors) {
                        forumDescriptorAdapter.add(fd);
                    }
                    busyIndicator.dismiss();
                    forumDescriptorAdapter.notifyDataSetChanged();
                }
            }.execute();

            getListView().setOnItemClickListener(new OnItemClickListener() {
                @Override
                public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                    final ForumDescriptor selected = (ForumDescriptor) getListView().getItemAtPosition(position);
                    List<ForumDescriptor> subforums = selected.getSubForums();
                    if (subforums.isEmpty()) {
                        // if we don't have any subs any more we want to list threads
                        Intent intent = new Intent(D2JspMobileActivity.this, ThreadListActivity.class);
                        intent.putExtra(FORUM_URL_KEY, selected.getURL());
                        startActivityForResult(intent, THREAD_LIST_ID);
                    } else {
                        Intent intent = new Intent(D2JspMobileActivity.this, D2JspMobileActivity.class);
                        intent.putExtra(SUBFORUMS_KEY, subforums.toArray(new ForumDescriptor[subforums.size()]));
                        startActivityForResult(intent, THREAD_LIST_ID);
                    }
                }
            });
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        // save cookies when this activity goes out of scope
        JSoupCookieManager.flushToDisk(getApplicationContext());
    }

    ////////////////////////////////////////////////////////////
    //	Private Stuff

    private ForumDescriptor[] getViewableForums() {
        Bundle extras = getIntent().getExtras();
        if (extras != null) {
            Parcelable[] subs = extras.getParcelableArray(SUBFORUMS_KEY);
            if (subs != null) {
                ForumDescriptor[] subforums = new ForumDescriptor[subs.length];
                for (int i = 0; i < subs.length; i++) {
                    subforums[i] = (ForumDescriptor) subs[i];
                }
                return subforums;
            }
        }

        return new ForumParser().getInfoDescriptors();
    }

    ////////////////////////////////////////////////////////////
    //	Inner Classes

    private final class ForumDescriptorAdapter extends ArrayAdapter<ForumDescriptor> {

        ////////////////////////////////////////////////////////////
        //	Constructor

        public ForumDescriptorAdapter(Context context, int textViewResourceId, List<ForumDescriptor> descriptors) {
            super(context, textViewResourceId, descriptors);
        }

        ////////////////////////////////////////////////////////////
        //	Overridden

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            View v = convertView;
            if (v == null) {
                LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                v = vi.inflate(R.layout.list_item, null);
            }
            ForumDescriptor fd = getItem(position);
            if (fd != null) {
                TextView forumNameView = (TextView) v.findViewById(R.id.forum_name);
                if (forumNameView != null) {
                    forumNameView.setText(fd.getName());
                }
            }
            return v;
        }

    }
}