package com.bananahammock.activities;

import android.app.Activity;
import android.content.Intent;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.*;
import com.bananahammock.R;
import com.bananahammock.Utils;
import com.bananahammock.auth.D2JspLoginHelper;
import com.bananahammock.auth.JSoupCookieManager;
import com.bananahammock.auth.LoginToken;
import org.apache.http.cookie.Cookie;

import java.io.IOException;

/**
 * @author Scott Faria
 */
public final class LoginActivity extends Activity {

    ////////////////////////////////////////////////////////////////////
    // Statics

    private static int LOGIN_ACTIVITY_ID = 101;

    ////////////////////////////////////////////////////////////////////
    // Variables

    private LoginToken currentLoginToken;

    ////////////////////////////////////////////////////////////////////
    // Overridden

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.login);

        TextView userNameTextView = (TextView) findViewById(R.id.username);
        userNameTextView.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View view, boolean isFocused) {
                if (!isFocused) {
                    resetSecurityImage();
                }
            }
        });

        resetSecurityImage();

        Button loginButton = (Button) findViewById(R.id.login_button);
        loginButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                LoginToken token = currentLoginToken;
                if (token != null) {
                    String uid = token.getUID();
                    String magicKey = token.getMagicKey();
                    byte[] password = ((EditText) findViewById(R.id.password)).getText().toString().getBytes();
                    String secId = token.getSecurityId();
                    String securityCode = ((EditText) findViewById(R.id.security_image_value)).getText().toString();
                    boolean showOnline = ((CheckBox) findViewById(R.id.show_in_lists)).isChecked();
                    Cookie[] cookies = D2JspLoginHelper.getAuthCookies(uid, magicKey, password, secId, securityCode, showOnline);
                    if (cookies == null) {
                        // no cookies == no login, the user information is probably wrong
                        Toast.makeText(getApplicationContext(), "Login failed! Please check your username/password/security image and try again.", Toast.LENGTH_SHORT).show();
                        resetSecurityImage();
                    } else {
                        for (Cookie cookie : cookies) {
                            JSoupCookieManager.instance().addCookie(cookie);
                        }

                        Intent intent = new Intent(LoginActivity.this, D2JspMobileActivity.class);
                        startActivityForResult(intent, LOGIN_ACTIVITY_ID);
                    }
                }
            }
        });
    }

    ////////////////////////////////////////////////////////////////////
    // private stuff

    private void resetSecurityImage() {
        TextView userNameTextView = (TextView) findViewById(R.id.username);
        String username = userNameTextView.getText().toString();
        new AsyncTask<String, Integer, Drawable>() {
            @Override
            protected Drawable doInBackground(String... username) {
                try {
                    LoginToken token = D2JspLoginHelper.getUsernameResponse(username[0]);
                    currentLoginToken = token;
                    return Utils.createDrawableFromURL(token.getSecurityImageURL());
                } catch (IOException e) {
                    Log.e("Log in failed", "Unable to get log in information from the server.", e);
                }

                return null;
            }

            @Override
            protected void onPostExecute(Drawable image) {
                ProgressBar progress = (ProgressBar) findViewById(R.id.security_image_progress);
                ImageView securityImage = (ImageView) findViewById(R.id.security_image);
                if (image == null) {
                    securityImage.setImageDrawable(null);
                    progress.setVisibility(View.VISIBLE);
                } else {
                    securityImage.setImageDrawable(image);
                    progress.setVisibility(View.GONE);
                }
            }
        }.execute(username);
    }
}

