package com.bananahammock.activities;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.*;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import com.bananahammock.R;
import com.bananahammock.shared.InformationProvider;
import com.bananahammock.thread.PostDescriptor;
import com.bananahammock.thread.ThreadParser;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: 11/7/11
 * Time: 10:20 PM
 * To change this template use File | Settings | File Templates.
 */
public class ThreadActivity extends ListActivity {

    ////////////////////////////////////////////////////////////
    //	Variables

    private int currentPage = 0;

    ////////////////////////////////////////////////////////////
    //	Overridden

    @Override
    protected final void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setTitle(getIntent().getStringExtra(D2JspMobileActivity.THREAD_NAME_KEY));

        final PostDescriptorAdapter postDescriptorAdapter = new PostDescriptorAdapter(
				this,
				R.layout.thread,
				new ArrayList<PostDescriptor>()
		);

		setListAdapter(postDescriptorAdapter);
        populateThread(postDescriptorAdapter);
    }

    @Override
    public final boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.thread_menu, menu);
        return true;
    }

    @Override
    public final boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.page_forward:
                currentPage++;
                populateThread((PostDescriptorAdapter) getListAdapter());
                return true;
            case R.id.page_back:
                Math.max(--currentPage, 0);
                populateThread((PostDescriptorAdapter) getListAdapter());
                return true;
            default:
                return false;
        }
    }

    ////////////////////////////////////////////////////////////
	//	Private Methods

    private void populateThread(final PostDescriptorAdapter postDescriptorAdapter) {
        //start up the busy indicator
        final ProgressDialog busyIndicator = ProgressDialog.show(this, "Please wait...", "Retrieving Posts...", true);

        //populate and inform the client we are waiting on it
        new AsyncTask<Bundle, Integer, PostDescriptor[]>() {
            @Override
            protected PostDescriptor[] doInBackground(Bundle... bundles) {
                Bundle extras = bundles[0];
                if (extras == null) {
                    throw new RuntimeException("Unable to get the thread url from the forum activity");
                }
                String threadUrl = extras.getString(D2JspMobileActivity.THREAD_URL_KEY);

                return new ThreadParser(threadUrl, currentPage).getInfoDescriptors();
            }
            @Override
            protected void onPostExecute(PostDescriptor[] postDescriptors) {
                postDescriptorAdapter.clear();
                for (PostDescriptor pd : postDescriptors) {
                    postDescriptorAdapter.add(pd);
                }
                busyIndicator.dismiss();
                postDescriptorAdapter.notifyDataSetChanged();
                setSelection(0);
            }
        }.execute(getIntent().getExtras());
    }

    ////////////////////////////////////////////////////////////
	//	Inner Classes

    private final class PostDescriptorAdapter extends ArrayAdapter<PostDescriptor> {

        ////////////////////////////////////////////////////////////
		//	Constructor

		public PostDescriptorAdapter(Context context, int textViewResourceId, List<PostDescriptor> descriptors) {
			super(context, textViewResourceId, descriptors);
		}

        @Override
        public View getView(int position, View v, ViewGroup parent) {
			if (v == null) {
				LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
				v = vi.inflate(R.layout.thread, null);
			}
			PostDescriptor pd = getItem(position);
			if (pd != null) {
				TextView posterNameView = (TextView) v.findViewById(R.id.poster_name);
				if (posterNameView != null) {
					posterNameView.setText(pd.getPosterName());
				}

                WebView contentView = (WebView) v.findViewById(R.id.post_content);
                if (contentView != null) {
                    contentView.getSettings().setPluginState(WebSettings.PluginState.ON);
                    contentView.getSettings().setJavaScriptEnabled(true);
                    contentView.loadDataWithBaseURL("file:///android_asset/", pd.getPostContent(), "text/html", "UTF-8", null);
				}
			}
			return v;
        }
    }
}
