package com.bananahammock.activities;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.TextView;
import com.bananahammock.R;
import com.bananahammock.Utils;
import com.bananahammock.shared.InformationProvider;
import com.bananahammock.thread.ThreadDescriptor;
import com.bananahammock.thread.ThreadListParser;

import java.util.ArrayList;
import java.util.List;

/**
 * Sub-activity for listing all threads in a forum/subforum
 *
 * @author Scott Faria - scott.faria@gmail.com
 */
public final class ThreadListActivity extends ListActivity {

    ////////////////////////////////////////////////////////////
    //	Overridden

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        final ThreadDescriptorAdapter threadDescriptorAdapter = new ThreadDescriptorAdapter(
                this,
                R.layout.thread_list_item,
                new ArrayList<ThreadDescriptor>()
        );

        setListAdapter(threadDescriptorAdapter);

        //start up the busy indicator
        final ProgressDialog busyIndicator = ProgressDialog.show(this, "Please wait...", "Retrieving threads...", true);

        //populate and inform the client we are waiting on it
        new AsyncTask<Bundle, Integer, ThreadDescriptor[]>() {
            @Override
            protected ThreadDescriptor[] doInBackground(Bundle... bundles) {
                Bundle extras = bundles[0];
                if (extras == null) {
                    throw new RuntimeException("Unable to get the thread url from the forum activity");
                }
                String forumUrl = extras.getString(D2JspMobileActivity.FORUM_URL_KEY);
                return getThreads(forumUrl);
            }

            @Override
            protected void onPostExecute(ThreadDescriptor[] threadDescriptors) {
                threadDescriptorAdapter.notifyDataSetInvalidated();
                for (ThreadDescriptor td : threadDescriptors) {
                    threadDescriptorAdapter.add(td);
                }
                busyIndicator.dismiss();
                threadDescriptorAdapter.notifyDataSetChanged();
            }
        }.execute(getIntent().getExtras());

        getListView().setOnItemClickListener(new OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                ThreadDescriptor selected = (ThreadDescriptor) getListView().getItemAtPosition(position);
                Intent intent = new Intent(ThreadListActivity.this, ThreadActivity.class);
                intent.putExtra(D2JspMobileActivity.THREAD_URL_KEY, selected.getURL());
                intent.putExtra(D2JspMobileActivity.THREAD_NAME_KEY, selected.getTitle());
                startActivityForResult(intent, 103);
            }
        });
    }

    ////////////////////////////////////////////////////////////
    //	Private Stuff

    private ThreadDescriptor[] getThreads(String forumUrl) {
        // TODO need to implement navigating thread pages
        InformationProvider<ThreadDescriptor> threadProvider = new ThreadListParser(forumUrl, 0);
        return threadProvider.getInfoDescriptors();
    }

    ////////////////////////////////////////////////////////////
    //	Inner Classes

    private final class ThreadDescriptorAdapter extends ArrayAdapter<ThreadDescriptor> {

        ////////////////////////////////////////////////////////////
        //	Constructor

        public ThreadDescriptorAdapter(Context context, int textViewResourceId, List<ThreadDescriptor> descriptors) {
            super(context, textViewResourceId, descriptors);
        }

        ////////////////////////////////////////////////////////////
        //	Overridden

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            View v = convertView;
            if (v == null) {
                LayoutInflater vi = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                v = vi.inflate(R.layout.thread_list_item, null);
            }
            ThreadDescriptor td = getItem(position);
            if (td != null) {
                TextView threadTitleView = (TextView) v.findViewById(R.id.thread_name);
                if (threadTitleView != null) {
                    threadTitleView.setText(td.getTitle());
                }

                TextView authorView = (TextView) v.findViewById(R.id.author);
                if (authorView != null) {
                    authorView.setText(td.getAuthor());
                }
            }
            return v;
        }

    }
}