package com.bananahammock.auth;

import android.os.AsyncTask;
import android.util.Log;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.cookie.Cookie;
import org.apache.http.impl.client.DefaultHttpClient;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Attributes;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;


/**
 * @author Scott Faria
 */
public final class D2JspLoginHelper {

	// //////////////////////////////////////////////////////////////////
	// Static Variables

	private static final String BASE_URL = "http://forums.d2jsp.org/";

	private static final String LOGIN_C2 = BASE_URL + "login.php?c=2&username=%s";

	private static final String LOGIN_C3 = BASE_URL + "login.php?c=3&referer=&uid=%s&key=%s&pass=%s&secId=%s&tSecCode=%s&stayLogged=1&showOnline=%s";


	////////////////////////////////////////////////////////////////////
	// Statics

    public static final LoginToken getUsernameResponse(String username) throws IOException {
            String postURL = String.format(LOGIN_C2, username);
			Document loginDoc = Jsoup.connect(postURL)
									 .data("query", "Java")
									 .userAgent("Mozilla")
									 .post();

			String uid = null;
			String magicKey = null;
			String secId = null;
			String secImgUrl = null;

			Elements loginContent = loginDoc.getElementsByTag("form");
			for (Element element : loginContent) {
				if (element.hasAttr("name") && element.attr("name").equals("a")) {
					Elements idElements = element.getElementsByTag("input");
					for (Element idElement : idElements) {
						if (idElement.hasAttr("name")) {
							Attributes attributes = idElement.attributes();
							String name = attributes.get("name");
							if (name.equals("uid")) {
								uid = attributes.get("value");
							} else if (name.equals("key")) {
								magicKey = attributes.get("value");
							} else if (name.equals("secId")) {
								secId = attributes.get("value");
							}
						}
					}

					Elements tableElements = element.getElementsByClass("ftbt");
					for (Element tableElement : tableElements) {
						Elements imgElements = tableElement.getElementsByTag("img");
						for (Element imgElement : imgElements) {
							Attributes attributes = imgElement.attributes();
							String imgUrl = attributes.get("src");
							if (imgUrl.startsWith("si.php?")) {
								secImgUrl = BASE_URL + imgUrl;
							}
						}
					}
				}
			}
        return new DefaultLoginToken(uid, magicKey, secId, secImgUrl);

    }

    public static Cookie[] getAuthCookies(String uid, String magicKey, byte[] password, String secId, String securityCode, boolean showOnline) {
        try {
            String loginURL = String.format(LOGIN_C3, uid, magicKey, new String(password), secId, securityCode, showOnline ? "1" : "0");
            return new AsyncTask<String, Integer, Cookie[]>() {
                @Override
                protected Cookie[] doInBackground(String... urls) {
                    try {
                        return getCookies(urls[0]);
                    } catch (IOException e) {
                        Log.d("D2JspLoginHelper.java", "Failed to get cookies from the server.", e);
                    }
                    return null;
                }
            }.execute(loginURL).get(30, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            Log.d("D2jspLoginHelper", "Task interrupted while getting cookies from server.", e);
        } catch (ExecutionException e) {
            Log.e("D2jspLoginHelper", "Unexpected exception while getting cookies from server.", e);
        } catch (TimeoutException e) {
            Log.e("D2jspLoginHelper", "Timed out while getting cookies from server.", e);
        }

        return null;
    }

	// //////////////////////////////////////////////////////////////////
	// Private Methods

	private static Cookie[] getCookies(String loginUrl) throws IOException {
		DefaultHttpClient client = new DefaultHttpClient();
	    client.getParams().setParameter("http.useragent", "Mozilla");
	    HttpGet get = new HttpGet(loginUrl);
	    HttpResponse response = client.execute(get);
        HttpEntity entity = response.getEntity();
        boolean isValidLogin = isValidReturnResult(entity.getContent());

        if (entity != null) {
            entity.consumeContent();
        }

        if (isValidLogin == false) {
            return null;
        }

        List<Cookie> cookies = client.getCookieStore().getCookies();
        return cookies.toArray(new Cookie[0]);
	}

    private static boolean isValidReturnResult(InputStream content) {
        try {
            Document doc = Jsoup.parse(content, "utf-8", BASE_URL);
            for (Element element : doc.getElementsByTag("legend")) {
                String val = element.text();
                if (val != null && val.equals("Log In Error")) {
                    return false;
                }
            }
            return true;
        } catch (IOException ex) {
            Log.e(D2JspLoginHelper.class.getSimpleName(), "There was a problem determining whether the log in was successful", ex);
        }
        return false;
    }

	////////////////////////////////////////////////////////////////////
	// Constructors

	private D2JspLoginHelper() {}

}

