package com.bananahammock.foruminfo;

import android.os.Parcel;
import android.os.Parcelable;
import com.bananahammock.Utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: 9/10/11
 * Time: 7:52 PM
 * To change this template use File | Settings | File Templates.
 */
final class DefaultForumDescriptor implements ForumDescriptor {

    ////////////////////////////////////////////////////////////
	//	Statics

    private static final String SELF_ITEM_NAME_PREFIX = "Continue to: ";

    public static final Parcelable.Creator<ForumDescriptor> CREATOR = new Parcelable.Creator<ForumDescriptor>() {
        public ForumDescriptor createFromParcel(Parcel in) {
            return new DefaultForumDescriptor(in);
        }

        public ForumDescriptor[] newArray(int size) {
            return new ForumDescriptor[size];
        }
    };

    ////////////////////////////////////////////////////////////
	//	Variables

    private final String name;
    private final String url;
    private final List<ForumDescriptor> subforums;

    ////////////////////////////////////////////////////////////
	//	Constructors

    DefaultForumDescriptor(String name, String url, List<ForumDescriptor> subforums) {
        this.name = Utils.checkNull(name);
        this.url = Utils.checkNull(url);
        this.subforums = Utils.checkNull(subforums);
    }

    DefaultForumDescriptor(Parcel parcel) {
        this.name = parcel.readString();
        this.url = parcel.readString();
        Parcelable[] subs = parcel.readParcelableArray(ClassLoader.getSystemClassLoader());
        this.subforums = new ArrayList<ForumDescriptor>(subs.length);
        for (Parcelable sub : subs) {
            subforums.add((ForumDescriptor) sub);
        }
    }

    ////////////////////////////////////////////////////////////
	//	Overridden

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getURL() {
        return url;
    }

    @Override
    public String toString() {
        return getName();
    }

    public List<ForumDescriptor> getSubForums() {
        List<ForumDescriptor> subs = new ArrayList<ForumDescriptor>(subforums);
        if (subs.isEmpty() == false) {
            subs.add(new DefaultForumDescriptor(SELF_ITEM_NAME_PREFIX + name, url, Collections.<ForumDescriptor>emptyList()));
        }
        return subs;
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(Parcel parcel, int i) {
        parcel.writeString(name);
        parcel.writeString(url);
        parcel.writeParcelableArray(subforums.toArray(new Parcelable[0]), 0);
    }

}
