package com.bananahammock.foruminfo;

import com.bananahammock.JSoupUtils;
import com.bananahammock.shared.InformationProvider;
import org.jsoup.Connection;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Parses out the main page of the forums
 *
 * @author Scott Faria - scott.faria@gmail.com
 */
public final class ForumParser implements InformationProvider<ForumDescriptor> {

    ////////////////////////////////////////////////////////////
    //	Statics

    private static final String SUBFORUM_URL = "http://forums.d2jsp.org";

    ////////////////////////////////////////////////////////////
    //	Public

    @Override
    public ForumDescriptor[] getInfoDescriptors() {
        List<ForumDescriptor> forums = new ArrayList<ForumDescriptor>();
        try {
            //slurp down the doc from the forums
            Connection c = Jsoup.connect(SUBFORUM_URL)
                    .data("query", "Java")
                    .userAgent("Mozilla")
                    .timeout(3000);
            c = JSoupUtils.setCookies(c);
            Document doc = c.post();

            for (Element forumTable : doc.getElementsByClass("ftb")) {
                for (Element forumTd : forumTable.getElementsByTag("tr")) {
                    forums.addAll(parseForums(forumTd));
                }
            }

        } catch (IOException e) {
            throw new RuntimeException("Failed to connect to: " + SUBFORUM_URL, e);
        }

        return forums.toArray(new ForumDescriptor[forums.size()]);
    }

    ////////////////////////////////////////////////////////////
    //	Private Stuff

    private List<ForumDescriptor> parseForums(Element forumTd) {
        List<ForumDescriptor> forums = new ArrayList<ForumDescriptor>();
        boolean isInSubForums = false;
        List<ForumDescriptor> subs = new ArrayList<ForumDescriptor>();
        for (Element link : forumTd.getElementsByTag("a")) {
            String forumUrlFragment = link.attr("href");
            if (forumUrlFragment.startsWith("forum")) {
                String name = link.text();
                if (isInSubForums == false) {
                    subs = new ArrayList<ForumDescriptor>();
                    forums.add(new DefaultForumDescriptor(name, buildUrl(forumUrlFragment), subs));
                    isInSubForums = true;
                } else {
                    subs.add(new DefaultForumDescriptor(name, buildUrl(forumUrlFragment), Collections.<ForumDescriptor>emptyList()));
                }
            }
        }
        return forums;
    }

    private String buildUrl(String urlFragment) {
        return SUBFORUM_URL + "/" + urlFragment;
    }
}
