package com.bananahammock.thread;

import android.util.Log;
import com.bananahammock.JSoupUtils;
import com.bananahammock.shared.InformationProvider;
import org.jsoup.Connection;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: 9/9/11
 * Time: 11:10 PM
 * To change this template use File | Settings | File Templates.
 */
public final class ThreadListParser implements InformationProvider<ThreadDescriptor> {

    ////////////////////////////////////////////////////////////
    //	Statics

    private static final int THREADS_PER_PAGE = 25;

    ////////////////////////////////////////////////////////////
    //	Variables

    private final String subforumUrl;

    ////////////////////////////////////////////////////////////
    //	Constructors

    public ThreadListParser(String subforumUrl, int page) {
        if (page < 0) {
            throw new RuntimeException("The page index must be greater than 0 (" + page + ")");
        }
        this.subforumUrl = subforumUrl + "&o=" + (page * THREADS_PER_PAGE);
    }

    ////////////////////////////////////////////////////////////
    //	Overridden

    @Override
    public ThreadDescriptor[] getInfoDescriptors() {
        List<ThreadDescriptor> threads = new ArrayList<ThreadDescriptor>();

        try {
            Connection c = Jsoup.connect(subforumUrl)
                    .data("query", "Java")
                    .userAgent("Mozilla")
                    .timeout(3000);
            c = JSoupUtils.setCookies(c);
            Document doc = c.post();

            for (Element table : doc.getElementsByClass("ftb")) {
                for (Element row : table.getElementsByTag("tr")) {
                    Elements cols = row.getElementsByTag("td");
                    String threadUrl = null;
                    String threadTitle = null;
                    Iterator<Element> it = cols.iterator();
                    if (it.hasNext()) {
                        // skip the div
                        it.next();

                        Element threadLinkCol = it.next();
                        for (Element link : threadLinkCol.getElementsByTag("a")) {
                            String url = link.attr("href");
                            if (url.startsWith("topic")) {
                                String title = link.attr("title");
                                if (title.equals("Goto last post") == false) {
                                    threadUrl = url;
                                    threadTitle = link.text();
                                    break;
                                }
                            }

                        }
                    }


                    if (threadUrl != null && threadTitle != null) {
                        // the next is the OP
                        Element posterCol = it.next();
                        Element posterElement = posterCol.getElementsByTag("a").iterator().next();
                        String poster = posterElement.text();

                        threads.add(new DefaultThreadDescriptor(threadTitle, threadUrl, poster));
                    }
                }

            }
        } catch (Exception ex) {
            Log.e(ThreadListParser.class.getSimpleName(), "Failed to get list of threads from the server.", ex);
        }

        return threads.toArray(new ThreadDescriptor[threads.size()]);
    }

    public static void main(String[] args) {
        String url = "http://forums.d2jsp.org/forum.php?f=188";
        ThreadListParser parser = new ThreadListParser(url, 0);
        ThreadDescriptor[] threads = parser.getInfoDescriptors();
        for (ThreadDescriptor thread : threads) {
            System.err.println(thread.toString());
        }
        System.err.println("Total threads: " + threads.length);
    }
}
