package com.bananahammock.thread;

import android.util.Log;
import com.bananahammock.JSoupUtils;
import com.bananahammock.shared.InformationProvider;
import org.jsoup.Connection;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;
import org.jsoup.select.Elements;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: scott
 * Date: 9/15/11
 * Time: 11:18 PM
 * To change this template use File | Settings | File Templates.
 */
public final class ThreadParser implements InformationProvider<PostDescriptor> {

    ////////////////////////////////////////////////////////////
    //	Variables

    private static final int POSTS_PER_PAGE = 10;

    private final String threadUrl;

    ////////////////////////////////////////////////////////////
    //	Constructor

    public ThreadParser(String threadUrl, int page) {
        if (page < 0) {
            throw new RuntimeException("The page index must be greater than 0 (" + page + ")");
        }
        this.threadUrl = "http://forums.d2jsp.org/" + threadUrl + "&o=" + (page * POSTS_PER_PAGE);

    }

    ////////////////////////////////////////////////////////////
    //	Overridden

    @Override
    public PostDescriptor[] getInfoDescriptors() {
        List<PostDescriptor> posts = new ArrayList<PostDescriptor>();

        try {
            Connection c = Jsoup.connect(threadUrl)
                    .data("query", "Java")
                    .userAgent("Mozilla")
                    .timeout(3000);
            c = JSoupUtils.setCookies(c);
            Document doc = c.post();

            for (Element postDl : doc.getElementsByTag("dl")) {
                String posterName = getPosterName(postDl.getElementsByTag("dt"));
                // the post content is in a <td> class="lc"
                Elements lcElements = postDl.getElementsByClass("lc");
                if (lcElements.isEmpty() == false) {
                    Element postElement = lcElements.get(0);
                    Element postContentElement = postElement.getElementsByTag("div").get(0);
                    String postContent = wrapPostConent(postContentElement);
                    posts.add(new DefaultPostDescriptor(posterName, postContent));
                }
            }

        } catch (IOException ex) {
            Log.e("Failed to download posts.", "Failed to download posts for url: " + threadUrl, ex);
        }

        return posts.toArray(new PostDescriptor[posts.size()]);
    }

    ////////////////////////////////////////////////////////////
    //	Private Methods

    private String wrapPostConent(Element postContentElement) {
        String postContent = postContentElement.html();
        StringBuilder sb = new StringBuilder();
        sb.append(postContent);
        return sb.toString();
    }
    private String getPosterName(Elements dts) {
        // there should be only a single <dt> element that contains the poster name
        Element posterNameElement = dts.get(0);
        Element aElement = posterNameElement.getElementsByTag("a").get(0);
        Node posterNameNode = aElement.childNodes().get(0);
        return posterNameNode.attr("text");
    }

    ////////////////////////////////////////////////////////////
    //	Main

    public static void main(String[] args) {
        String threadUrl = "http://forums.d2jsp.org/topic.php?t=52288848&f=188";
        ThreadParser parser = new ThreadParser(threadUrl, 0);

        PostDescriptor[] posts = parser.getInfoDescriptors();
        for (PostDescriptor post : posts) {
            System.err.println(post);
        }
    }


}
